#! /usr/bin/env fan
//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   10 Jun 08  Brian Frank  Creation
//

using fwt

**
** FwtDemo displays the FWT sampler program.
**
class FwtDemo : Test
{

  **
  ** Put the whole thing together in a tabbed pane
  **
  Void main()
  {
    Window
    {
      title = "FWT Demo"
      bounds = Rect { x = 100; y = 100; w = 800; h = 600 }
      menuBar = makeMenuBar
      content = EdgePane
      {
        top = makeToolBar
        center = TabPane
        {
          Tab { text = "Buttons";        InsetPane { makeButtons } }
          Tab { text = "Labels";         InsetPane { makeLabels } }
          Tab { text = "WebBrowser";     InsetPane { makeWebBrowser } }
          Tab { text = "Text";           InsetPane { makeText} }
          Tab { text = "EdgePane";       InsetPane { makeEdgePane } }
          Tab { text = "GridPane";       InsetPane { makeGridPane } }
          Tab { text = "Tree and Table"; InsetPane { makeTreeAndTable } }
          Tab { text = "Serialization";  InsetPane { makeSerialization } }
          Tab { text = "Graphics";       InsetPane { makeGraphics } }
        }
      }
    }.open
  }

  **
  ** Build the menu bar
  **
  Menu makeMenuBar()
  {
    return Menu
    {
      Menu
      {
        text = "File";
        MenuItem { text = "Back";    image = backIcon;    onAction = &browser.back }
        MenuItem { text = "Next";    image = nextIcon;    onAction = &browser.forward }
        MenuItem { text = "Refresh"; image = refreshIcon; onAction = &browser.refresh }
        MenuItem { text = "Stop";    image = stopIcon;    onAction = &browser.stop }
        MenuItem { text = "Exit"; onAction = |,| { Sys.exit } }
      }

      Menu
      {
        text = "Nested";
        Menu
        {
          text = "Alpha"
          image = folderIcon
          MenuItem { text = "Alpha.1"; onAction = &cb }
          Menu
          {
            text = "Alpha.2"
            MenuItem { text = "Alpha.2.I"; onAction = &cb }
            Menu
            {
              text = "Alpha.2.II"
              MenuItem { text = "Alpha.2.II.a"; onAction = &cb }
              MenuItem { text = "Alpha.2.II.b"; onAction = &cb }
            }
            MenuItem { text = "Alpha.2.III"; onAction = &cb }
          }
        }
        Menu
        {
          text = "Beta"
          MenuItem { text = "Beta.1"; onAction = &cb }
          MenuItem { text = "Beta.2"; onAction = &cb }
        }
      }

      Menu
      {
        text = "Modes"
        MenuItem { text = "Check 1"; mode = MenuItemMode.check; onAction = &cb }
        MenuItem { text = "Check 2"; mode = MenuItemMode.check; onAction = &cb }
        MenuItem { mode = MenuItemMode.sep }
        MenuItem { text = "Radio 1"; mode = MenuItemMode.radio; onAction = &cb }
        MenuItem { text = "Radio 2"; mode = MenuItemMode.radio; onAction = &cb; selected = true  }
      }

    }
  }

  **
  ** Build the toolbar
  **
  Widget makeToolBar()
  {
    return ToolBar
    {
      Button { image = backIcon;    onAction = &browser.back }
      Button { image = nextIcon;    onAction = &browser.forward }
      Button { image = refreshIcon; onAction = &browser.refresh }
      Button { image = stopIcon;    onAction = &browser.stop }
      Button { mode  = ButtonMode.sep }
      Button { image = sysIcon;   mode = ButtonMode.check; onAction = &cb }
      Button { image = prefsIcon; mode = ButtonMode.toggle; onAction = &cb }
      Button { mode  = ButtonMode.sep }
      Button { image = audioIcon; mode = ButtonMode.radio; onAction = &cb; selected = true }
      Button { image = imageIcon; mode = ButtonMode.radio; onAction = &cb; }
      Button { image = videoIcon; mode = ButtonMode.radio; onAction = &cb; }
    }
  }

  **
  ** Build a simple web browser
  **
  Widget makeWebBrowser()
  {
    url := Text { text=homeUri }
    url.onAction = |,| { browser.load(url.text.toUri) }

    return EdgePane
    {
      top = EdgePane { center=url; right=Label{text="Enter to Go!"} }
      center = browser
    }
  }

  **
  ** Build a pane of various labels
  **
  Widget makeLabels()
  {
    return GridPane
    {
      numCols = 2
      hgap = 20
      Label { text = "Text Only" }
      Label { image = stopIcon }
      Label { text = "Both"; image = folderIcon }
      Label { text = "Colors"; image = folderIcon; fg = Color.red; bg = Color.yellow }
      Label { text = "Left"; halign = Halign.left }
      Label { text = "Center"; halign = Halign.center }
      Label { text = "Right"; halign = Halign.right }
    }
  }

  **
  ** Build a pane of various buttons
  **
  Widget makeButtons()
  {
    return GridPane
    {
      numCols = 3
      hgap = 20
      Button { text = "B1"; image = stopIcon; onAction = &cb }
      Button { text = "Button 2"; onAction = &cb }
      Button { mode = ButtonMode.toggle; text = "Button 3"; onAction = &cb }
      Button { mode = ButtonMode.check; text = "B4"; onAction = &cb }
      Button { mode = ButtonMode.radio; text = "Button 5"; onAction = &cb }
      Button { mode = ButtonMode.radio; text = "B6"; onAction = &cb }
      Button { text = "Popup 1"; onAction = &popup(true) }
      Button { text = "Popup 2"; onAction = &popup(false) }
    }
  }

  **
  ** Build a pane of various text fields
  **
  Widget makeText()
  {
    area := Text
    {
      multiLine = true
      font = Font { name = "Courier"; size=10 }
      text ="Press button above to serialize this entire demo here"
    }

    ecb := |Event e| { echo("onAction: \"${e.widget->text}\"") }
    ccb := |Event e| { echo("onModify: \"${e.widget->text}\"") }

    nums := ["One", "Two", "Three", "Four", "Five", "Six", "Seven" ]

    return EdgePane
    {
      left = GridPane
      {
        numCols = 2

        Label { text="Single" }
        Text { onAction=ecb; onAction=ecb; onModify=ccb }

        Label { text="Courier";  }
        Text { font = Font { name ="Courier" }; onAction=ecb; onModify=ccb  }

        Label { text="Password" }
        Text { password = true; onAction=ecb; onAction=ecb; onModify=ccb }

        Label { text="Combo" }
        Combo { items=nums; onAction=ecb; onModify=ccb }

        Label { text="Combo editable=false" }
        Combo { editable=true; items=nums; onAction=ecb; onModify=ccb }

        Label { text="Combo dropDown=false" }
        Combo { dropDown=false; items=nums; onAction=ecb; onModify=ccb }

        Label { text="MultiLine" }

        Button { text="Serialize Demo"; onAction=&serializeTo(area) }
      }
      center = InsetPane.make(5) { content=area }
    }
  }

  Void serializeTo(Text area)
  {
    try
    {
      opts := ["indent":2, "skipDefaults":true, "skipErrors":true]
      buf := Buf.make.writeObj(area.window, opts)
      area.text = buf.flip.readAllStr
    }
    catch (Err e)
    {
      area.text = e.traceToStr
    }
  }

  **
  ** Build a demo edge pane
  **
  Widget makeEdgePane()
  {
    return EdgePane
    {
      top    = Button { text = "top" }
      left   = Button { text = "left" }
      right  = Button { text = "right" }
      bottom = Button { text = "bottom" }
      center = Button { text = "center" }
    }
  }

  **
  ** Build a demo grid pane using randomly sized boxes
  **
  Widget makeGridPane()
  {
    grid := GridPane
    {
      numCols = 5
      hgap = 10
      vgap = 10
      Box { color = Color.red }
      Box { color = Color.green }
      Box { color = Color.yellow }
      Box { color = Color.blue }
      Box { color = Color.orange }
      Box { color = Color.darkGray }
      Box { color = Color.purple }
      Box { color = Color.gray }
      Box { color = Color.white }
    }
    colors := [Color.red, Color.green, Color.yellow, Color.blue, Color.orange,
               Color.darkGray, Color.purple, Color.gray, Color.white]

    15.times |Int i| { grid.add(Box { color=colors[i%colors.size] }) }

    controls := GridPane
    {
      numCols = 2
      halignCells = Halign.fill
      Label { text="numCols" };      Text { text="5"; onModify=&setInt(grid, "numCols") }
      Label { text="hgap" };         Text { text="10"; onModify=&setInt(grid, "hgap") }
      Label { text="vgap" };         Text { text="10"; onModify=&setInt(grid, "vgap") }
      Label { text="halignCells" };  Combo { items=Halign.values; onModify=&setEnum(grid, "halignCells") }
      Label { text="valignCells" };  Combo { items=Valign.values; onModify=&setEnum(grid, "valignCells") }
      Label { text="halignPane" };   Combo { items=Halign.values; onModify=&setEnum(grid, "halignPane") }
      Label { text="valignPane" };   Combo { items=Valign.values; onModify=&setEnum(grid, "valignPane") }
      Label { text="expandRow" };    Text { text="null"; onModify=&setInt(grid, "expandRow") }
      Label { text="expandCol" };    Text { text="null"; onModify=&setInt(grid, "expandCol") }
    }

    return EdgePane { left=controls; center=InsetPane { content=grid } }
  }

  **
  ** Build a demo tree and table for file system
  **
  Widget makeTreeAndTable()
  {
    tree := Tree
    {
      model = DirTreeModel { demo = this }
    }

    table := Table
    {
      model = DirTableModel { demo = this; dir = File.os(".").list }
    }

    updateTable := |File dir| { table.model->dir = dir.list; table.updateAll }
    tree.onAction = |Event e| { updateTable(e.data) }
    table.onAction = |Event e| { updateTable(table.model->dir->get(e.index)) }

    return SashPane.make.add(tree).add(table)
  }

  **
  ** Build a pane showing how to use serialization
  **
  Widget makeSerialization()
  {
    area := Text
    {
      multiLine = true
      font = Font { name = "Courier"; size=10 }
      text =
        "fwt::EdgePane\n" +
        "{\n" +
        "  top = fwt::Button { text=\"Top\" }\n" +
        "  center = fwt::Button { text=\"Center\" }\n" +
        "  bottom = fwt::Button { text=\"Bottom\" }\n" +
        "}\n"
    }

    test := InsetPane
    {
      Label { text="Press button to deserialize code on the left here (doesn't work yet)" }
    }

    return SashPane
    {
      //weights = [30,70]
      EdgePane
      {
        center = area
        right = InsetPane
        {
          Button { text="=>"; onAction=|,| { deserializeTo(area.text, test) } }
        }
      }
      add(test)
    }
  }

  Void deserializeTo(Str text, InsetPane test)
  {
    try
    {
      test.content = InStream.makeForStr(text).readObj
    }
    catch (Err e)
    {
      test.content = Text { multiLine = true; text = e.traceToStr }
    }
    test.relayout
  }

  **
  ** Build a pane showing how to use Graphics
  **
  Widget makeGraphics()
  {
    return ScrollPane { content=GraphicsDemo { demo = this } }
  }

  static Void setInt(Widget obj, Str field, Event e)
  {
    f := obj.type.field(field)
    Str text := e.widget->text
    int := text.toInt(10, false)
    if (int != null || text=="null") f.set(obj, int)
    obj.relayout
  }

  static Void setEnum(Widget obj, Str field, Event e)
  {
    f := obj.type.field(field)
    en := f.get(obj)->fromStr(e.widget->text, false)
    if (en != null) f.set(obj, en)
    obj.relayout
  }

  static Void cb(Event event)
  {
    w := event.widget
    echo("${w->text} selected=${w->selected}")
  }

  static Void popup(Bool withPos, Event event)
  {
    Menu
    {
      MenuItem { text = "Popup 1"; onAction = &cb }
      MenuItem { text = "Popup 2"; onAction = &cb }
      MenuItem { text = "Popup 3"; onAction = &cb }
    }.open(event.widget, withPos ? Point.make(0, event.widget.size.h) : null)
  }

  WebBrowser browser := WebBrowser {}
  Str homeUri := "http://fandev.org/"

  File scriptDir  := File.make(type->sourceFile.toStr.toUri).parent

  Uri:File icons    := Pod.find("icons").files
  Image backIcon    := Image.make(icons[`/x16/go-previous.png`])
  Image nextIcon    := Image.make(icons[`/x16/go-next.png`])
  Image cutIcon     := Image.make(icons[`/x16/edit-cut.png`])
  Image copyIcon    := Image.make(icons[`/x16/edit-copy.png`])
  Image pasteIcon   := Image.make(icons[`/x16/edit-paste.png`])
  Image folderIcon  := Image.make(icons[`/x16/folder.png`])
  Image fileIcon    := Image.make(icons[`/x16/text-x-generic.png`])
  Image audioIcon   := Image.make(icons[`/x16/audio-x-generic.png`])
  Image imageIcon   := Image.make(icons[`/x16/image-x-generic.png`])
  Image videoIcon   := Image.make(icons[`/x16/video-x-generic.png`])
  Image sysIcon     := Image.make(icons[`/x16/applications-system.png`])
  Image prefsIcon   := Image.make(icons[`/x16/preferences-system.png`])
  Image refreshIcon := Image.make(icons[`/x16/view-refresh.png`])
  Image stopIcon    := Image.make(icons[`/x16/process-stop.png`])
}

**************************************************************************
** DirTreeModel
**************************************************************************

class DirTreeModel : TreeModel
{
  FwtDemo demo

  override Obj[] roots() { return Sys.homeDir.listDirs }

  override Str text(Obj node) { return node->name }

  override Image image(Obj node) { return demo.folderIcon }

  override Obj[] children(Obj obj) { return obj->listDirs }
}

**************************************************************************
** DirTableModel
**************************************************************************

class DirTableModel : TableModel
{
  FwtDemo demo
  File[] dir
  Str[] headers := ["Name", "Size", "Modified"]
  override Int numCols() { return 3 }
  override Int numRows() { return dir.size }
  override Str header(Int col) { return headers[col] }
  override Halign halign(Int col) { return col == 1 ? Halign.right : Halign.left }
  override Str text(Int col, Int row)
  {
    f := dir[row]
    switch (col)
    {
      case 0:  return f.name
      case 1:  s := f.size; return s == null ? "" : s/1024 +"KB"
      case 2:  return f.modified.toLocale
      default: return "?"
    }
  }
  override Image image(Int col, Int row)
  {
    if (col != 0) return null
    return dir[row].isDir ? demo.folderIcon : demo.fileIcon
  }
}

**************************************************************************
** Box
**************************************************************************

class Box : Widget
{
  Color color := Color.green

  override Size prefSize := Size.make(Int.random(20..100), Int.random(20..80))

  override Void onPaint(Graphics g)
  {
    size := this.size
    g.brush = color
    g.fillRect(0, 0, size.w, size.h)
    g.brush = Color.black
    g.drawRect(0, 0, size.w-1, size.h-1)
  }
}

**************************************************************************
** GraphicsDemo
**************************************************************************

class GraphicsDemo : Widget
{
  FwtDemo demo

  override Size prefSize(Hints hints := null) { return Size.make(600,500) }

  override Void onPaint(Graphics g)
  {
    w := size.w
    h := size.h

    g.antialias = true

    g.brush = Gradient.makeLinear(
      Point.make(0,0), Color.white,
      Point.make(w,h), Color.make(0x66_66_66))
    g.fillRect(0, 0, w, h)

    g.brush = Color.black; g.drawRect(0, 0, w-1, h-1)

    g.brush = Color.black
    w.times |Int i| { if (i.isOdd) g.drawPoint(i, 2) }
    w.times |Int i| { if (i.isEven) g.drawPoint(i, 3) }

    g.brush = Color.orange; g.fillRect(10, 10, 50, 60)
    g.brush = Color.blue; g.drawRect(10, 10, 50, 60)

    g.brush = Color.yellow; g.fillOval(40, 40, 120, 100)
    g.pen = Pen { width = 2; dash=[8,4].toImmutable }
    g.brush = Color.green; g.drawOval(40, 40, 120, 100)

    g.pen = Pen { width = 8; join = Pen.joinBevel }
    g.brush = Color.gray; g.drawRect(120, 120, 120, 90)
    g.brush = Color.orange; g.fillArc(120, 120, 120, 90, 45, 90)
    g.pen = Pen { width = 8; cap = Pen.capRound }
    g.brush = Color.blue; g.drawArc(120, 120, 120, 90, 45, 90)

    g.brush = Color.purple; g.drawText("Hello World!", 70, 50)
    g.font = Font { name="Courier"; size=16; bold=true }; g.drawText("Hello World!", 70, 70)

    img := demo.folderIcon
    g.drawImage(img, 220, 30)
    g.copyImage(img, Rect { x=0; y=0; w=img.size.w; h=img.size.h }, Rect { x=250; y=30; w=64; h=64})

    g.brush = Gradient.makeLinear(
      Point.make(300,120), Color.blue,
      Point.make(300+200,120+200), Color.red)
    g.pen = Pen { width=20; join = Pen.joinRound }
    g.drawRect(310, 130, 180, 180)
    6.times |Int i| { g.drawText("Gradients!", 340, 150+i*20) }
  }
}
