//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   12 Jun 08  Brian Frank  Creation
//
package fan.fwt;

import fan.sys.*;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.swt.events.*;

public class ButtonPeer
  extends WidgetPeer
  implements SelectionListener
{
  public Widget create(Widget parent)
  {
    fan.fwt.Button self = (fan.fwt.Button)this.self;
    if (parent instanceof ToolBar)
    {
      ToolItem b = new ToolItem((ToolBar)parent, mode(self.mode, true));
      control = b;
      setSelected(selected);
      b.addSelectionListener(this);
      return b;
    }
    else
    {
      Button b = new Button((Composite)parent, mode(self.mode, false));
      control = b;
      setSelected(selected);
      b.addSelectionListener(this);
      return b;
    }
  }

  public Obj send(Str id, Obj arg)
  {
    if (id == fan.fwt.Button.getSelectedId) return getSelected();
    if (id == fan.fwt.Button.setSelectedId) return setSelected((Bool)arg);
    return super.send(id, arg);
  }

  Bool getSelected()
  {
    if (control == null) { return selected; }
    if (control instanceof Button)
      return Bool.make(((Button)control).getSelection());
    else
      return Bool.make(((ToolItem)control).getSelection());
  }

  Obj setSelected(Bool sel)
  {
    if (control == null) { selected = sel; return null; }
    if (control instanceof Button)
      ((Button)control).setSelection(sel.val);
    else
      ((ToolItem)control).setSelection(sel.val);
    return null;
  }

  public void sync(Str f)
  {
    Env env = Env.get();
    fan.fwt.Button self = (fan.fwt.Button)this.self;

    if (this.control instanceof Button)
    {
      Button c = (Button)this.control;
      if (f == null || f == self.textId)   c.setText(self.text().val);
      if (f == null || f == self.imageId)  c.setImage(env.image(self.image()));
      if (f == null || f == self.fontId)   c.setFont(env.font(self.font()));
    }
    else
    {
      ToolItem c = (ToolItem)this.control;
      if (f == null || f == self.textId)   c.setText(self.text().val);
      if (f == null || f == self.imageId)  c.setImage(env.image(self.image()));
    }
  }

  public void widgetSelected(SelectionEvent event)
  {
    fan.fwt.Button self = (fan.fwt.Button)this.self;
    Func cb = self.onAction();
    if (cb != null) cb.call1(event(event));
  }

  public void widgetDefaultSelected(SelectionEvent event)
  {
    // not used
  }

  static int mode(ButtonMode mode, boolean toolbar)
  {
    if (mode == ButtonMode.push)   return SWT.PUSH;
    if (mode == ButtonMode.check)  return SWT.CHECK;
    if (mode == ButtonMode.toggle) return toolbar ? SWT.CHECK : SWT.TOGGLE;
    if (mode == ButtonMode.radio)  return SWT.RADIO;
    if (mode == ButtonMode.sep)    return SWT.SEPARATOR;
    throw new IllegalStateException(""+mode);
  }

  Bool selected = Bool.False;
}