//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   16 Jun 08  Brian Frank  Creation
//
package fan.fwt;

import fan.sys.*;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.swt.events.*;

public class MenuItemPeer
  extends WidgetPeer
  implements SelectionListener
{

  public Widget create(Widget parent)
  {
    // the parent of a MenuItem is always a Menu which
    // has both a Menu and MenuItem SWT control, we always
    // want to use the Menu as our parent
    Menu parentMenu = ((MenuPeer)self.parent().peer).menu;

    fan.fwt.MenuItem self = (fan.fwt.MenuItem)this.self;
    MenuItem m = new MenuItem(parentMenu, mode(self.mode));
    control = m;
    setSelected(selected);
    m.addSelectionListener(this);
    return m;
  }

  public Obj send(Str id, Obj arg)
  {
    if (id == fan.fwt.MenuItem.getSelectedId) return getSelected();
    if (id == fan.fwt.MenuItem.setSelectedId) return setSelected((Bool)arg);
    return super.send(id, arg);
  }

  Bool getSelected()
  {
    if (!(control instanceof MenuItem)) { return selected; }
    return Bool.make(((MenuItem)control).getSelection());
  }

  Obj setSelected(Bool sel)
  {
    if (!(control instanceof MenuItem)) { return selected; }
    ((MenuItem)control).setSelection(sel.val);
    return null;
  }

  public void sync(Str f)
  {
    Env env = Env.get();
    fan.fwt.MenuItem self = (fan.fwt.MenuItem)this.self;
    MenuItem c = (MenuItem)this.control;

    if (f == null || f == self.textId)  c.setText(self.text().val);
    if (f == null || f == self.imageId) c.setImage(env.image(self.image()));
  }

  public void widgetSelected(SelectionEvent event)
  {
    fan.fwt.MenuItem self = (fan.fwt.MenuItem)this.self;
    Func cb = self.onAction();
    if (cb != null) cb.call1(event(event));
  }

  public void widgetDefaultSelected(SelectionEvent event)
  {
    // not used
  }

  static int mode(MenuItemMode mode)
  {
    if (mode == MenuItemMode.push)  return SWT.PUSH;
    if (mode == MenuItemMode.check) return SWT.CHECK;
    if (mode == MenuItemMode.radio) return SWT.RADIO;
    if (mode == MenuItemMode.sep)   return SWT.SEPARATOR;
    throw new IllegalStateException(""+mode);

  }

  Bool selected = Bool.False;
}