//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   16 Jun 08  Brian Frank  Creation
//
package fan.fwt;

import fan.sys.*;
import fan.sys.List;
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Widget;

public class MenuPeer extends MenuItemPeer
{

  public Widget create(Widget parent)
  {
    // if my parent is a Menu, then create myself
    // as a cascade menu which is two SWT controls:
    //   this.control = MenuItem
    //   this.menu    = Menu
    if (self.parent() instanceof fan.fwt.Menu)
    {
      Menu parentMenu = (((MenuPeer)self.parent.peer).menu);
      menu = new Menu(parentMenu);
      MenuItem item = new MenuItem(parentMenu, SWT.CASCADE);
      item.setMenu(menu);
      return item;
    }

    // if my parent is a Shell, then create myself
    // as the menu bar which is one SWT control:
    //   this.control = this.menu = Menu
    if (parent instanceof Shell)
    {
      menu = new Menu((Shell)parent, SWT.BAR);
      ((Shell)parent).setMenuBar(menu);
      return menu;
    }

    throw new IllegalStateException("Unsupported parent for Menu: " + self.parent());
  }

  public Obj send(Str id, Obj arg)
  {
    if (id == fan.fwt.Menu.openId) { open((List)arg); return null; }
    return super.send(id, arg);
  }

  public void open(List args)
  {
    if (control != null)
      throw new IllegalStateException("Menu is already mounted");

    fan.fwt.Widget parent = (fan.fwt.Widget)args.get(0);
    fan.fwt.Point pos = (fan.fwt.Point)args.get(1);

    menu = new Menu((Control)parent.peer.control);
    attachTo(menu);
    if (pos != null) menu.setLocation(new Point((int)pos.x.val, (int)pos.y.val));
    menu.setVisible(true);

    Display display = Env.get().display;
    while (!menu.isDisposed() && menu.isVisible())
      if (!display.readAndDispatch())
        display.sleep();

    detach(self);
  }

  public void sync(Str f)
  {
    // if the control is a Menu, then nothing to sync,
    // otherwise sync text/image for cascade menu item
    if (control instanceof Menu) return;
    super.sync(f);
  }

  public void detach(fan.fwt.Widget self)
  {
    super.detach(self);
    if (menu != null) { menu.dispose(); menu = null; }
  }

  Menu menu;
}