//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   25 Jun 08  Brian Frank  Creation
//
package fan.fwt;

import fan.sys.*;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Widget;

public class TablePeer
  extends WidgetPeer
  implements Listener
{
  public Widget create(Widget parent)
  {
    Table t = new Table((Composite)parent, SWT.VIRTUAL | SWT.BORDER);
    t.addListener(SWT.SetData, this);
    t.addListener(SWT.DefaultSelection, this);
    this.control = t;
    rebuild();
    return t;
  }

  public Obj send(Str id, Obj arg)
  {
    if (id == fan.fwt.Table.updateAllId) { updateAll(); return null; }
    return super.send(id, arg);
  }

  void updateAll()
  {
    Table c = (Table)this.control;

    TableModel model = model();
    if (model == null) return;

    c.removeAll();
    c.setItemCount((int)model.numRows().val);
    c.clearAll();
  }

  public void sync(Str f)
  {
    //Env env = Env.get();
    fan.fwt.Table self = (fan.fwt.Table)this.self;
    Table c = (Table)this.control;

    if (f == null || f == self.headerVisibleId) c.setHeaderVisible(self.headerVisible().val);
  }

  public void handleEvent(Event event)
  {
    switch (event.type)
    {
      case SWT.SetData:          handleSetData(event); break;
      case SWT.DefaultSelection: handleDefaultSelection(event); break;
      default: System.out.println("WARNING: TreePeer.handleEvent: " + event);
    }
  }

  private void handleSetData(Event event)
  {
    TableModel model = model();
    if (model == null) return;

    Env env = Env.get();
    TableItem item = (TableItem)event.item;

    int numCols = (int)model.numCols().val;
    Int row = Int.make(event.index);
    for (int i=0; i<numCols; ++i)
    {
      Int col = Int.make(i);
      item.setText(i, model.text(col, row).val);
      item.setImage(i, env.image(model.image(col, row)));
    }
  }

  private void handleDefaultSelection(Event event)
  {
    Table table = (Table)this.control;
    fan.fwt.Table self = (fan.fwt.Table)this.self;
    TableItem item = (TableItem)event.item;
    if (self.onAction != null)
    {
      fan.fwt.Event fevent = event(event);
      fevent.index = Int.make(table.getSelectionIndex());
      self.onAction.call1(fevent);
    }
  }

  public void rebuild()
  {
    // TODO: need to figure out how to sync
    Table table = (Table)this.control;

    // get model
    TableModel model = model();
    if (model == null) return;

    // build columns
    int numCols = (int)model.numCols().val;
    for (int i=0; i<numCols; ++i)
    {
      Int col = Int.make(i);
      TableColumn tc = new TableColumn(table, style(model.halign(col)));
      tc.setText(model.header(col).val);
      tc.setWidth(100);
    }

    // rows
    table.setItemCount((int)model.numRows().val);
  }

  public TableModel model()
  {
    return ((fan.fwt.Table)this.self).model;
  }

  int curNumCol = -1;
}