//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   12 Jun 08  Brian Frank  Creation
//
package fan.fwt;

import fan.sys.*;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.swt.events.*;

public class TextPeer
  extends WidgetPeer
  implements Listener, ModifyListener
{

  public Widget create(Widget parent)
  {
    fan.fwt.Text self = (fan.fwt.Text)this.self;

    int style = self.multiLine.val ? SWT.MULTI: SWT.SINGLE;
    if (self.border.val)   style |= SWT.BORDER;
    if (self.wrap.val)     style |= SWT.WRAP;
    if (self.password.val) style |= SWT.PASSWORD;
    if (self.hscroll.val)  style |= SWT.H_SCROLL;
    if (self.vscroll.val)  style |= SWT.V_SCROLL;

    Text t = new Text((Composite)parent, style);
    control = t;
    t.addListener(SWT.DefaultSelection, this);
    t.addModifyListener(this);
    setText(text);
    return t;
  }

  public Obj send(Str id, Obj arg)
  {
    if (id == fan.fwt.Text.getTextId) return getText();
    if (id == fan.fwt.Text.setTextId) return setText((Str)arg);
    return super.send(id, arg);
  }

  Str getText()
  {
    if (control == null) { return text; }
    return Str.make(((Text)control).getText());
  }

  Obj setText(Str t)
  {
    if (control == null) { text = t; return null; }
    ((Text)control).setText(t.val);
    return null;
  }

  public void sync(Str f)
  {
    Env env = Env.get();
    fan.fwt.Text self = (fan.fwt.Text)this.self;
    Text c = (Text)this.control;

    if (f == null || f == self.fontId)  c.setFont(env.font(self.font()));
  }

  public void handleEvent(Event event)
  {
    fan.fwt.Text self = (fan.fwt.Text)this.self;
    if (event.type == SWT.DefaultSelection)
    {
      if (self.onAction != null) self.onAction.call1(event(event));
    }
  }

  public void modifyText(ModifyEvent event)
  {
    fan.fwt.Text self = (fan.fwt.Text)this.self;
    if (self.onModify != null) self.onModify.call1(event(event));
  }

  Str text = Str.Empty;
}