//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   20 Jul 08  Brian Frank  Creation
//
package fan.fwt;

import fan.sys.*;
import fan.sys.List;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.Widget;

public class TreePeer
  extends WidgetPeer
  implements Listener
{
  public Widget create(Widget parent)
  {
    Tree t = new Tree((Composite)parent, SWT.VIRTUAL | SWT.BORDER);
    t.addListener(SWT.SetData, this);
    t.addListener(SWT.DefaultSelection, this);
    this.control = t;
    rebuild();
    return t;
  }

  public Obj send(Str id, Obj arg)
  {
    if (id == fan.fwt.Tree.updateAllId) { updateAll(); return null; }
    return super.send(id, arg);
  }

  void updateAll()
  {
    Tree c = (Tree)this.control;

    TreeModel model = model();
    if (model == null) return;

    c.removeAll();
    c.setItemCount(model.roots().sz());
    c.clearAll(true);
  }

  public void sync(Str f)
  {
    //Env env = Env.get();
    //fan.fwt.Tree self = (fan.fwt.Tree)this.self;
    //Tree c = (Tree)this.control;
  }

  public void handleEvent(Event event)
  {
    switch (event.type)
    {
      case SWT.SetData:          handleSetData(event); break;
      case SWT.DefaultSelection: handleDefaultSelection(event); break;
      default: System.out.println("WARNING: TreePeer.handleEvent: " + event);
    }
  }

  private void handleSetData(Event event)
  {
    TreeModel model = model();
    if (model == null) return;

    Env env = Env.get();
    TreeItem item = (TreeItem)event.item;
    TreeItem parentItem = item.getParentItem();

    // root
    Obj node;
    if (parentItem == null)
      node = model.roots().get(event.index);
    else
      node = ((Data)parentItem.getData()).children.get(event.index);

    Data data = new Data();
    data.node = node;
    data.children = model.children(node);

    List kids = model.children(node);
    item.setText(model.text(node).val);
    item.setImage(env.image(model.image(node)));
    item.setData(data);
    item.setItemCount(kids.sz());
  }

  private void handleDefaultSelection(Event event)
  {
    fan.fwt.Tree self = (fan.fwt.Tree)this.self;
    TreeItem item = (TreeItem)event.item;
    Data data = (Data)item.getData();
    if (self.onAction == null)
    {
      item.setExpanded(!item.getExpanded());
    }
    else
    {
      fan.fwt.Event fevent = event(event);
      fevent.data = data.node;
      self.onAction.call1(fevent);
    }
  }

  public void rebuild()
  {
    // TODO: need to figure out how to sync
    Tree tree = (Tree)this.control;

    // get model
    TreeModel model = model();
    if (model == null) return;

    // define roots
    tree.setItemCount(model.roots().sz());
  }

  public TreeModel model()
  {
    return ((fan.fwt.Tree)this.self).model;
  }

  static class Data
  {
    Obj node;
    List children;
  }
}