//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   12 Jun 08  Brian Frank  Creation
//
package fan.fwt;

import java.util.HashMap;
import fan.sys.*;
import fan.sys.List;
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.*;

/**
 * Native methods for Widget
 */
public class WidgetPeer
  implements PaintListener
{

//////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////

  public static WidgetPeer make(fan.fwt.Widget self)
    throws Exception
  {
    // first most specific peer class
    Type t = self.type();
    Class cls = null;
    while (true)
    {
      cls = (Class)peers.get(t.qname().val);
      if (cls != null) break;
      t = t.base();
    }

    // instantiate
    WidgetPeer peer = (WidgetPeer)cls.newInstance();
    peer.self = self;
    return peer;
  }

  static HashMap peers = new HashMap();
  static
  {
    peers.put("fwt::Button",     ButtonPeer.class);
    peers.put("fwt::Combo",      ComboPeer.class);
    peers.put("fwt::Label",      LabelPeer.class);
    peers.put("fwt::MenuItem",   MenuItemPeer.class);
    peers.put("fwt::Menu",       MenuPeer.class);
    peers.put("fwt::Pane",       PanePeer.class);
    peers.put("fwt::SashPane",   SashPanePeer.class);
    peers.put("fwt::ScrollPane", ScrollPanePeer.class);
    peers.put("fwt::RichText",   RichTextPeer.class);
    peers.put("fwt::TabPane",    TabPanePeer.class);
    peers.put("fwt::Tab",        TabPeer.class);
    peers.put("fwt::Table",      TablePeer.class);
    peers.put("fwt::Text",       TextPeer.class);
    peers.put("fwt::ToolBar",    ToolBarPeer.class);
    peers.put("fwt::Tree",       TreePeer.class);
    peers.put("fwt::Widget",     WidgetPeer.class);
    peers.put("fwt::Window",     WindowPeer.class);
    peers.put("fwt::WebBrowser", WebBrowserPeer.class);
  }

//////////////////////////////////////////////////////////////////////////
// Accessors
//////////////////////////////////////////////////////////////////////////

  public fan.fwt.Widget parent()
  {
    return self.parent();
  }

  public Widget parentControl()
  {
    fan.fwt.Widget p = self.parent();
    return p == null ? null : p.peer.control;
  }

//////////////////////////////////////////////////////////////////////////
// State
//////////////////////////////////////////////////////////////////////////

  public final Bool enabled(fan.fwt.Widget self)
  {
    // pain in the ass due to SWT's class hierarchy
    if (control instanceof Control)  return enabled = Bool.make(((Control)control).getEnabled());
    if (control instanceof MenuItem) return enabled = Bool.make(((MenuItem)control).getEnabled());
    if (control instanceof ToolItem) return enabled = Bool.make(((ToolItem)control).getEnabled());
    return enabled;
  }

  public final void enabled(fan.fwt.Widget self, Bool b)
  {
    // pain in the ass due to SWT's class hierarchy
    enabled = b;
    if (control instanceof Control)  ((Control)control).setEnabled(b.val);
    if (control instanceof MenuItem) ((MenuItem)control).setEnabled(b.val);
    if (control instanceof ToolItem) ((ToolItem)control).setEnabled(b.val);
  }

  public final Bool visible(fan.fwt.Widget self)
  {
    if (!(control instanceof Control)) { return visible; }
    return visible = Bool.make(((Control)control).getVisible());
  }

  public final void visible(fan.fwt.Widget self, Bool b)
  {
    visible = b;
    if (control instanceof Control)
      ((Control)control).setVisible(b.val);
  }

//////////////////////////////////////////////////////////////////////////
// Layout
//////////////////////////////////////////////////////////////////////////

  public final fan.fwt.Point pos(fan.fwt.Widget self)
  {
    if (!(control instanceof Control)) return pos;
    return point( ((Control)control).getLocation() );
  }

  public final void pos(fan.fwt.Widget self, fan.fwt.Point pos)
  {
    if (!(control instanceof Control)) { this.pos = pos; return; }
    ((Control)control).setLocation((int)pos.x.val, (int)pos.y.val);
  }

  public final Size size(fan.fwt.Widget self)
  {
    if (!(control instanceof Control)) return size;
    return size( ((Control)control).getSize() );
  }

  public final void size(fan.fwt.Widget self, Size size)
  {
    if (!(control instanceof Control)) { this.size = size; return; }
    ((Control)control).setSize((int)size.w.val, (int)size.h.val);
  }

  public final Rect bounds(fan.fwt.Widget self)
  {
    if (!(control instanceof Control)) return Rect.make(pos.x, pos.y, size.w, size.h);
    return rect( ((Control)control).getBounds() );
  }

  public final void bounds(fan.fwt.Widget self, Rect b)
  {
    if (!(control instanceof Control)) { pos = b.pos(); size = b.size(); return; }
    ((Control)control).setBounds((int)b.x.val, (int)b.y.val, (int)b.w.val, (int)b.h.val);
  }

  public Size prefSize(fan.fwt.Widget self, Hints hints)
  {
    if (!(control instanceof Control)) return Size.def;
    int w = (hints.w == null) ? SWT.DEFAULT : (int)hints.w.val;
    int h = (hints.h == null) ? SWT.DEFAULT : (int)hints.h.val;
    Point s = ((Control)control).computeSize(w, h, true);
    return size(s);
  }

  public fan.fwt.Point posOnDisplay(fan.fwt.Widget self)
  {
    if (!(control instanceof Control)) return null;
    Point pt = Env.get().display.map((Control)control, null, 0, 0);
    return point(pt);
  }

  public void relayout()
  {
    if (control instanceof Composite)
      ((Composite)control).layout(true);
  }

  public void repaint(Rect r)
  {
    if (control instanceof Control)
    {
      Control c = (Control)control;
      if (r == null)
        c.redraw();
      else
        c.redraw((int)r.x.val, (int)r.y.val, (int)r.w.val, (int)r.h.val, true);
    }
  }

//////////////////////////////////////////////////////////////////////////
// Attachment
//////////////////////////////////////////////////////////////////////////

  public final Bool attached(fan.fwt.Widget self)
  {
    return control != null ? Bool.True : Bool.False;
  }

  public final void attach(fan.fwt.Widget self)
  {
    // short circuit if I'm already attached
    if (control != null) return;

    // short circuit if my parent isn't attached
    fan.fwt.Widget parentWidget = self.parent();
    if (parentWidget == null || parentWidget.peer.control == null) return;

    // create control and initialize
    // TODO: need to rework this cluster f**k
    Widget parentControl = parentWidget.peer.control;
    if (parentControl instanceof TabItem)
    {
      TabItem item = (TabItem)parentControl;
      attachTo(create(item.getParent()));
      item.setControl((Control)this.control);
    }
    else
    {
      attachTo(create(parentControl));
      if (parentControl instanceof ScrolledComposite)
        ((ScrolledComposite)parentControl).setContent((Control)control);
    }
  }

  void attachTo(Widget control)
  {
    // sync with native control
    this.control = control;
    if (pos != fan.fwt.Point.def) pos(self, pos);
    if (size != fan.fwt.Size.def) size(self, size);
    if (!enabled.val) enabled(self, enabled);
    if (!visible.val) visible(self, visible);
    sync(null);

    // recursively attach my children
    List kids = self.kids;
    for (int i=0; i<kids.sz(); ++i)
    {
      fan.fwt.Widget kid = (fan.fwt.Widget)kids.get(i);
      kid.peer.attach(kid);
    }
  }

  public Widget create(Widget parent)
  {
    Canvas c = new Canvas((Composite)parent, 0)
    {
      public void drawBackground(GC gc, int x, int y, int w, int h) {}
    };
    c.addPaintListener(this);
    return c;
  }

  public void detach(fan.fwt.Widget self)
  {
    if (control == null) return;
    control.dispose();
    control = null;
  }

//////////////////////////////////////////////////////////////////////////
// Widget/Control synchronization
//////////////////////////////////////////////////////////////////////////

  public final void sync(fan.fwt.Widget self, Str id)
  {
    if (control == null) return;
    sync(id);
  }

  public void sync(Str id)
  {
  }

  public final Obj send(fan.fwt.Widget self, Str id, Obj arg)
  {
    if (id == fan.fwt.Widget.relayoutId) { relayout(); return null; }
    if (id == fan.fwt.Widget.repaintId)  { repaint((Rect)arg); return null; }
    return send(id, arg);
  }

  public Obj send(Str id, Obj arg)
  {
    System.out.println(id);
    return null;
  }

//////////////////////////////////////////////////////////////////////////
// Callbacks
//////////////////////////////////////////////////////////////////////////

  public void paintControl(PaintEvent e)
  {
    Graphics g = new Graphics();
    g.peer.gc = e.gc;
    self.onPaint(g);
  }

//////////////////////////////////////////////////////////////////////////
// Utils
//////////////////////////////////////////////////////////////////////////

  static fan.fwt.Point point(Point pt)
  {
    return fan.fwt.Point.make(Int.make(pt.x), Int.make(pt.y));
  }

  static fan.fwt.Size size(Point pt)
  {
    return fan.fwt.Size.make(Int.make(pt.x), Int.make(pt.y));
  }

  static fan.fwt.Rect rect(Rectangle r)
  {
    return fan.fwt.Rect.make(Int.make(r.x), Int.make(r.y), Int.make(r.width), Int.make(r.height));
  }

  static int style(Halign halign)
  {
    if (halign == Halign.left) return SWT.LEFT;
    if (halign == Halign.center) return SWT.CENTER;
    if (halign == Halign.right) return SWT.RIGHT;
    throw new IllegalStateException(halign.toString());
  }

  static int orientation(Orientation orientation)
  {
    if (orientation == Orientation.horizontal) return SWT.HORIZONTAL;
    if (orientation == Orientation.vertical) return SWT.VERTICAL;
    throw new IllegalStateException(orientation.toString());
  }

  fan.fwt.Event event(Event x)
  {
    fan.fwt.Event f = fan.fwt.Event.make();
    f.widget(self);
    return f;
  }

  fan.fwt.Event event(TypedEvent x)
  {
    fan.fwt.Event f = fan.fwt.Event.make();
    f.widget(self);
    return f;
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  fan.fwt.Widget self;
  Widget control;
  Bool enabled = Bool.True;
  Bool visible = Bool.True;
  fan.fwt.Point pos = fan.fwt.Point.def;
  fan.fwt.Size size = fan.fwt.Size.def;

}
