//
// Copyright (c) 2008, John Sublett
// Licensed under the Academic Free License version 3.0
//
// History:
//   10 Feb 08  John Sublett  Creation
//

**
** IndexDef defines an index on one or more fields
** of a type.
**
const internal class IndexDef
{
  new make(HavenService haven, Str name, ColDef[] columns, Bool desc := false)
  {
    this.haven = haven
    this.name = name
    this.columns = columns.toImmutable
    this.desc = desc
  }

  static IndexDef merge(IndexDef[] defs)
  {
    cols := ColDef[,]
    HavenService haven := null
    Str name := null
    Bool desc := null
    defs.each |IndexDef def|
    {
      if (haven == null)
        haven = def.haven
      else if (haven != def.haven)
        throw HavenErr.make("Incompatible index definitions for merge.")

      if (name == null)
        name = def.name
      else if (name != def.name)
        throw HavenErr.make("Incompatible index definitions for merge: $def.name != $name")

      if (desc == null)
        desc = def.desc
      else if (desc != def.desc)
        throw HavenErr.make("Incompatible index definitions for merge: desc=$def.desc != desc=$desc")

      cols.addAll(def.columns)
    }

    return IndexDef.make(haven, name, cols, desc)
  }

//////////////////////////////////////////////////////////////////////////
// Fields
//////////////////////////////////////////////////////////////////////////

  **
  ** The haven instance that owns this index.
  **
  const HavenService haven

  **
  ** The name of the index in the database.
  **
  const Str name

  **
  ** The list of columns the are in this index.
  **
  const ColDef[] columns

  **
  ** If true, the index is sorted in descending order.
  ** Otherwise, the index is sorted in ascending order.
  **
  const Bool desc := false

}