//
// Copyright (c) 2008, John Sublett
// Licensed under the Academic Free License version 3.0
//
// History:
//   14 Jan 08  John Sublett  Creation
//

using sql

**
** ColDef for the Uri data type.
**
@havenCol=Uri#
const internal class UriCol : ColDef
{
  new make() : super.make() {}

  new makeForField(HavenService haven, Field field) : super(haven, field)
  {
    dialect := haven.dialect
    mRefType := (Type)field.facet("ref")
    if (mRefType != null)
    {
      refTableName = haven.tableName(mRefType)
      Field mRefField := null

      // find the key field, require
      // a single key for a ref
      mRefType.fields.each |Field f|
      {
        if (f.facet("key", false))
        {
          if (mRefField != null)
            throw HavenErr.make("Single key required for ref uri.")
          mRefField = f
        }
      }

      if (mRefField == null)
        throw HavenErr.make("Key property required for ref uri target type.")
      refFieldName = mRefField.qname

      target = haven.makeColumn(mRefField)
    }
    else
      len = field.facet("len", 128)

    if (mRefType != null) refType = mRefType.toImmutable
  }

  override Str toSql(Obj o)
  {
    method := refField.of.method("fromStr")
    key := method.call1(o->name)
    if (target != null)
      return target.toSql(key)
    else
      return "'" + ((Uri)o).encode + "'"
  }

  override Void setFieldFromRow(Obj obj, Row row, HavenNamespace ns)
  {
    field.set(obj, ns.idToUri(refType, target.rowValue(row, (Col)row.type.field(field.name))))
  }

  override Obj rowValue(Row row, Col col)
  {
    return target.rowValue(row, col)
  }

  override Obj paramValue(Obj o, Str:Obj params)
  {
    v := null
    if (target != null)
    {
      method := refField.of.method("fromStr")
      key := method.call1(o->name)
      v = target.paramValue(key)
    }
    else
      v = ((Uri)o).encode

    if (params != null) params[field.name] = v
    return v
  }

  override Str sqlType()
  {
    if (target != null)
      return target.sqlType
    else
      return "VARCHAR($len)"
  }

  Str foreignKey()
  {
    return "$refTableName($refField.name)"
  }

  const Int len
  private const Type refType
  private const Str refTableName
  private const Str refFieldName // todo: use field when Slot is immutable
  private const ColDef target

  private Field refField()
  {
    return Slot.findField(refFieldName, false)
  }
}