//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   18 May 08  Brian Frank  Creation
//
package fan.sys;

import java.util.*;

/**
 * SysNamespace manages the "/sys" URI namespace branch.
 */
final class SysNamespace
  extends Namespace
{

//////////////////////////////////////////////////////////////////////////
// Identity
//////////////////////////////////////////////////////////////////////////

  public Type type() { return Sys.SysNamespaceType; }

//////////////////////////////////////////////////////////////////////////
// Namespace
//////////////////////////////////////////////////////////////////////////

  public Obj get(Uri uri, Bool checked)
  {
    // relativize to myself
    uri = uri.relTo(this.uri());

    // /service/{qname}
    if (uri.path().sz() == 2 && uri.path().get(0).toString().equals("service"))
    {
      String qname = uri.path().get(1).toString();
      Thread t = Thread.findService(qname, false);
      if (t != null) return t;
    }

    if (!checked.val) return null;
    throw UnresolvedErr.make(uri).val;
  }

}