//
// Copyright (c) 2008, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   18 May 08  Brian Frank  Creation
//

using System;

namespace Fan.Sys
{
  /// <summary>
  /// SysNamespace manages the "/sys" URI namespace branch.
  /// </summary>
  internal sealed class SysNamespace : Namespace
  {

  //////////////////////////////////////////////////////////////////////////
  // Identity
  //////////////////////////////////////////////////////////////////////////

    public override Type type() { return Sys.SysNamespaceType; }

  //////////////////////////////////////////////////////////////////////////
  // Namespace
  //////////////////////////////////////////////////////////////////////////

    public override Obj get(Uri uri, Bool check)
    {
      // relativize to myself
      uri = uri.relTo(this.uri());

      // /service/{qname}
      if (uri.path().sz() == 2 && uri.path().get(0).ToString().Equals("service"))
      {
        String qname = uri.path().get(1).ToString();
        Thread t = Thread.findService(qname, false);
        if (t != null) return t;
      }

      if (!check.val) return null;
      throw UnresolvedErr.make(uri).val;
    }

  }
}