//
// Copyright (c) 2006, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   3 Mar 06  Brian Frank  Creation
//

**
** NativeTest
**
class NativeTest : NativeBase
{

  new make()
  {
  }

  new makeY(Int a, Int b)
  {
    y = a + b
  }

  native static Int doStaticA()
  native static Int doStaticB(Int x, Int y)

  native Int fX
  native override Int fA
  native override Str fV
  override Int fA2 := 0xab

  native Int getCtorY()

  native Int getPeerZ()
  native Void setPeerZ(Int z)

  native Str defs1(Str a := "a")
  native Str defs2(Str a := "a", Str b := "b")
  native Str defs3(Str a, Str b := "b", Str c := "c")

  native static Str sdefs1(Str a := "x")
  native static Str sdefs2(Str a := "x", Str b := "y")
  native static Str sdefs3(Str a, Str b := "y", Str c := "z")

  Void testFields()
  {
    verifyEq(fX, 6);     fX = 99;  verifyEq(fX, 99)
    verifyEq(fA, 444);   fA = 33;  verifyEq(fA, 33)
    verifyEq(fV, "fV");  fV = "?"; verifyEq(fV, "?")
    verifyEq(fA2, 0xab); fA2 = 12; verifyEq(fA2, 12)
  }

  Void testMethods()
  {
    // static methods
    verifyEq(doStaticA(), 2006)
    verifyEq(doStaticB(4, 3), 7)

    // verify native make happens before field init (1234)
    // and before constructor (11) code
    verifyEq(makeY(5, 6).getCtorY(), null)

    // set field in the peer
    setPeerZ(55)
    verifyEq(getPeerZ(), 55)
  }

  Void testParamDefaults()
  {
    // instance
    verifyEq(defs1, "a")
    verifyEq(defs1("A"), "A")

    verifyEq(defs2, "ab")
    verifyEq(defs2("A"), "Ab")
    verifyEq(defs2("A", "B"), "AB")

    verifyEq(defs3("A"), "Abc")
    verifyEq(defs3("A", "B"), "ABc")
    verifyEq(defs3("A", "B", "C"), "ABC")

    // static
    verifyEq(sdefs1, "x")
    verifyEq(sdefs1("A"), "A")

    verifyEq(sdefs2, "xy")
    verifyEq(sdefs2("A"), "Ay")
    verifyEq(sdefs2("A", "B"), "AB")

    verifyEq(sdefs3("A"), "Ayz")
    verifyEq(sdefs3("A", "B"), "ABz")
    verifyEq(sdefs3("A", "B", "C"), "ABC")
  }

  Void testSubclasses()
  {
    n := NativeSub.make
    /*
      TODO:
        - this code crashes HotSpot
        - how to handle native code in subclasses; two (or more) peers?

    verifyEq(n.getCtorY, 9);
    verifyEq(n.getMyX, 6)
    n.setMyX(71); verifyEq(n.getMyX, 71)
    n.setPeerZ(42)
    verifyEq(n.getPeerZ, 55)
    */
  }

  Int x := 6
  Int y := 1234

}

//////////////////////////////////////////////////////////////////////////
// NativeBase
//////////////////////////////////////////////////////////////////////////

abstract class NativeBase : Test
{
  abstract Int fA
  virtual Str fV
  abstract Int fA2
}

//////////////////////////////////////////////////////////////////////////
// NativeSub
//////////////////////////////////////////////////////////////////////////

class NativeSub : NativeTest
{
  new make() : super() {}
}